@@ -2,8 +2,6 @@
 
 from typing import ClassVar
 
-from pydantic import SecretStr
-
 from semantic_kernel.kernel_pydantic import KernelBaseSettings
 from semantic_kernel.utils.feature_stage_decorator import experimental
 
@@ -14,19 +12,16 @@ class AzureAIAgentSettings(KernelBaseSettings):
 
     Args:
         model_deployment_name: Azure AI Agent (Env var AZURE_AI_AGENT_MODEL_DEPLOYMENT_NAME)
-        project_connection_string: Azure AI Agent Project Connection String
-            (Env var AZURE_AI_AGENT_PROJECT_CONNECTION_STRING)
         endpoint: Azure AI Agent Endpoint (Env var AZURE_AI_AGENT_ENDPOINT)
-        subscription_id: Azure AI Agent Subscription ID (Env var AZURE_AI_AGENT_SUBSCRIPTION_ID)
-        resource_group_name: Azure AI Agent Resource Group Name (Env var AZURE_AI_AGENT_RESOURCE_GROUP_NAME)
-        project_name: Azure AI Agent Project Name (Env var AZURE_AI_AGENT_PROJECT_NAME)
+        api_version: Azure AI Agent API Version (Env var AZURE_AI_AGENT_API_VERSION)
     """
 
     env_prefix: ClassVar[str] = "AZURE_AI_AGENT_"
 
     model_deployment_name: str
-    project_connection_string: SecretStr | None = None
     endpoint: str | None = None
-    subscription_id: str | None = None
-    resource_group_name: str | None = None
-    project_name: str | None = None
+    agent_id: str | None = None
+    bing_connection_id: str | None = None
+    azure_ai_search_connection_id: str | None = None
+    azure_ai_search_index_name: str | None = None
+    api_version: str | None = None
