@@ -4,7 +4,7 @@
 import asyncio
 import logging
 import sys
-from collections.abc import AsyncIterable
+from collections.abc import AsyncIterable, Awaitable, Callable
 from functools import partial, reduce
 from typing import Any, ClassVar
 
@@ -15,7 +15,7 @@ if sys.version_info >= (3, 12):
 else:
     from typing_extensions import override  # pragma: no cover
 
-from semantic_kernel.agents.agent import AgentResponseItem, AgentThread
+from semantic_kernel.agents import AgentResponseItem, AgentThread
 from semantic_kernel.agents.bedrock.action_group_utils import (
     parse_function_result_contents,
     parse_return_control_payload,
@@ -44,6 +44,7 @@ from semantic_kernel.utils.feature_stage_decorator import experimental
 from semantic_kernel.utils.telemetry.agent_diagnostics.decorators import (
     trace_agent_get_response,
     trace_agent_invocation,
+    trace_agent_streaming_invocation,
 )
 
 logger = logging.getLogger(__name__)
@@ -207,7 +208,7 @@ class BedrockAgent(BedrockAgentBase):
             An instance of BedrockAgent with the created agent.
         """
         try:
-            bedrock_agent_settings = BedrockAgentSettings.create(
+            bedrock_agent_settings = BedrockAgentSettings(
                 agent_resource_role_arn=agent_resource_role_arn,
                 foundation_model=foundation_model,
                 env_file_path=env_file_path,
@@ -261,8 +262,8 @@ class BedrockAgent(BedrockAgentBase):
     @override
     async def get_response(
         self,
+        messages: str | ChatMessageContent | list[str | ChatMessageContent] | None = None,
         *,
-        input_text: str,
         thread: AgentThread | None = None,
         agent_alias: str | None = None,
         arguments: KernelArguments | None = None,
@@ -272,7 +273,7 @@ class BedrockAgent(BedrockAgentBase):
         """Get a response from the agent.
 
         Args:
-            input_text (str): The input text.
+            messages (str | ChatMessageContent | list[str | ChatMessageContent]): The messages.
             thread (AgentThread, optional): The thread. This is used to maintain the session state in the service.
             agent_alias (str, optional): The agent alias.
             arguments (KernelArguments, optional): The kernel arguments to override the current arguments.
@@ -282,8 +283,11 @@ class BedrockAgent(BedrockAgentBase):
         Returns:
             A chat message content with the response.
         """
+        if not isinstance(messages, str) and not isinstance(messages, ChatMessageContent):
+            raise AgentInvokeException("Messages must be a string or a ChatMessageContent for BedrockAgent.")
+
         thread = await self._ensure_thread_exists_with_messages(
-            messages=[input_text],
+            messages=messages,
             thread=thread,
             construct_thread=lambda: BedrockAgentThread(bedrock_runtime_client=self.bedrock_runtime_client),
             expected_type=BedrockAgentThread,
@@ -302,7 +306,7 @@ class BedrockAgent(BedrockAgentBase):
         kwargs.setdefault("sessionState", {})
 
         for _ in range(self.function_choice_behavior.maximum_auto_invoke_attempts):
-            response = await self._invoke_agent(thread.id, input_text, agent_alias, **kwargs)
+            response = await self._invoke_agent(thread.id, messages, agent_alias, **kwargs)
 
             events: list[dict[str, Any]] = []
             for event in response.get("completion", []):
@@ -355,9 +359,10 @@ class BedrockAgent(BedrockAgentBase):
     @override
     async def invoke(
         self,
-        input_text: str,
-        thread: AgentThread | None = None,
+        messages: str | ChatMessageContent | list[str | ChatMessageContent] | None = None,
         *,
+        thread: AgentThread | None = None,
+        on_new_message: Callable[[ChatMessageContent], Awaitable[None]] | None = None,
         agent_alias: str | None = None,
         arguments: KernelArguments | None = None,
         kernel: "Kernel | None" = None,
@@ -366,8 +371,9 @@ class BedrockAgent(BedrockAgentBase):
         """Invoke an agent.
 
         Args:
-            input_text (str): The input text.
+            messages (str | ChatMessageContent | list[str | ChatMessageContent]): The messages.
             thread (AgentThread, optional): The thread. This is used to maintain the session state in the service.
+            on_new_message: A callback function to handle intermediate steps of the agent's execution.
             agent_alias (str, optional): The agent alias.
             arguments (KernelArguments, optional): The kernel arguments to override the current arguments.
             kernel (Kernel, optional): The kernel to override the current kernel.
@@ -376,8 +382,14 @@ class BedrockAgent(BedrockAgentBase):
         Returns:
             An async iterable of chat message content.
         """
+        if not isinstance(messages, str) and not isinstance(messages, ChatMessageContent):
+            raise AgentInvokeException("Messages must be a string or a ChatMessageContent for BedrockAgent.")
+
+        if on_new_message:
+            logger.warning("The on_new_message callback is not supported for BedrockAgent.")
+
         thread = await self._ensure_thread_exists_with_messages(
-            messages=[input_text],
+            messages=messages,
             thread=thread,
             construct_thread=lambda: BedrockAgentThread(bedrock_runtime_client=self.bedrock_runtime_client),
             expected_type=BedrockAgentThread,
@@ -396,7 +408,7 @@ class BedrockAgent(BedrockAgentBase):
         kwargs.setdefault("sessionState", {})
 
         for _ in range(self.function_choice_behavior.maximum_auto_invoke_attempts):
-            response = await self._invoke_agent(thread.id, input_text, agent_alias, **kwargs)
+            response = await self._invoke_agent(thread.id, messages, agent_alias, **kwargs)
 
             events: list[dict[str, Any]] = []
             for event in response.get("completion", []):
@@ -447,13 +459,14 @@ class BedrockAgent(BedrockAgentBase):
             "Failed to get a response from the agent. Please consider increasing the auto invoke attempts."
         )
 
-    @trace_agent_invocation
+    @trace_agent_streaming_invocation
     @override
     async def invoke_stream(
         self,
-        input_text: str,
-        thread: AgentThread | None = None,
+        messages: str | ChatMessageContent | list[str | ChatMessageContent] | None = None,
         *,
+        thread: AgentThread | None = None,
+        on_new_message: Callable[[ChatMessageContent], Awaitable[None]] | None = None,
         agent_alias: str | None = None,
         arguments: KernelArguments | None = None,
         kernel: "Kernel | None" = None,
@@ -462,8 +475,10 @@ class BedrockAgent(BedrockAgentBase):
         """Invoke an agent with streaming.
 
         Args:
-            input_text (str): The input text.
+            messages (str | ChatMessageContent | list[str | ChatMessageContent]): The messages.
             thread (AgentThread, optional): The thread. This is used to maintain the session state in the service.
+            on_new_message: A callback function to handle intermediate steps of the
+                            agent's execution as fully formed messages.
             agent_alias (str, optional): The agent alias.
             arguments (KernelArguments, optional): The kernel arguments to override the current arguments.
             kernel (Kernel, optional): The kernel to override the current kernel.
@@ -472,8 +487,14 @@ class BedrockAgent(BedrockAgentBase):
         Returns:
             An async iterable of streaming chat message content
         """
+        if not isinstance(messages, str) and not isinstance(messages, ChatMessageContent):
+            raise AgentInvokeException("Messages must be a string or a ChatMessageContent for BedrockAgent.")
+
+        if on_new_message:
+            logger.warning("The on_new_message callback is not supported for BedrockAgent.")
+
         thread = await self._ensure_thread_exists_with_messages(
-            messages=[input_text],
+            messages=messages,
             thread=thread,
             construct_thread=lambda: BedrockAgentThread(bedrock_runtime_client=self.bedrock_runtime_client),
             expected_type=BedrockAgentThread,
@@ -492,7 +513,7 @@ class BedrockAgent(BedrockAgentBase):
         kwargs.setdefault("sessionState", {})
 
         for request_index in range(self.function_choice_behavior.maximum_auto_invoke_attempts):
-            response = await self._invoke_agent(thread.id, input_text, agent_alias, **kwargs)
+            response = await self._invoke_agent(thread.id, messages, agent_alias, **kwargs)
 
             all_function_call_messages: list[StreamingChatMessageContent] = []
             for event in response.get("completion", []):
