@@ -1,24 +1,29 @@
 # Copyright (c) Microsoft. All rights reserved.
-from typing import TYPE_CHECKING, Annotated
+
+import logging
+from typing import TYPE_CHECKING, Annotated, Any
+
+from semantic_kernel.functions.kernel_function_decorator import kernel_function
+from semantic_kernel.functions.kernel_function_from_prompt import KernelFunctionFromPrompt
 
 if TYPE_CHECKING:
     from semantic_kernel.functions.kernel_arguments import KernelArguments
     from semantic_kernel.kernel import Kernel
     from semantic_kernel.prompt_template.prompt_template_config import PromptTemplateConfig
 
+logger = logging.getLogger(__name__)
+
 
 class ConversationSummaryPlugin:
     """Semantic plugin that enables conversations summarization."""
 
-    from semantic_kernel.functions.kernel_function_decorator import kernel_function
-
     # The max tokens to process in a single semantic function call.
     _max_tokens = 1024
 
     _summarize_conversation_prompt_template = (
         "BEGIN CONTENT TO SUMMARIZE:\n{{"
-        + "$input"
-        + "}}\nEND CONTENT TO SUMMARIZE.\nSummarize the conversation in 'CONTENT TO"
+        "$input"
+        "}}\nEND CONTENT TO SUMMARIZE.\nSummarize the conversation in 'CONTENT TO"
         " SUMMARIZE',            identifying main points of discussion and any"
         " conclusions that were reached.\nDo not incorporate other general"
         " knowledge.\nSummary is in plain text, in complete sentences, with no markup"
@@ -26,17 +31,29 @@ class ConversationSummaryPlugin:
     )
 
     def __init__(
-        self, kernel: "Kernel", prompt_template_config: "PromptTemplateConfig", return_key: str = "summary"
+        self, prompt_template_config: "PromptTemplateConfig", return_key: str = "summary", **kwargs: Any
     ) -> None:
-        """Initializes a new instance of the ConversationSummaryPlugin class.
+        """Initializes a new instance of the ConversationSummaryPlugin.
+
+        The template for this plugin is built-in, and will overwrite any template passed in the prompt_template_config.
+
+        Args:
+            prompt_template_config (PromptTemplateConfig): The prompt template configuration.
+            return_key (str): The key to use for the return value.
+            **kwargs: Additional keyword arguments, not used only for compatibility.
 
-        :param kernel: The kernel instance.
-        :param prompt_template_config: The prompt template configuration.
-        :param return_key: The key to use for the return value.
         """
+        if "kernel" in kwargs:
+            logger.warning(
+                "The kernel parameter is not used in the ConversationSummaryPlugin constructor anymore."
+                "Please make sure to remove and to add the created plugin to the kernel, by using:"
+                "kernel.add_plugin(conversation_plugin, 'summarizer')"
+            )
+
         self.return_key = return_key
-        self._summarizeConversationFunction = kernel.add_function(
-            prompt=ConversationSummaryPlugin._summarize_conversation_prompt_template,
+        prompt_template_config.template = ConversationSummaryPlugin._summarize_conversation_prompt_template
+        prompt_template_config.template_format = "semantic-kernel"
+        self._summarizeConversationFunction = KernelFunctionFromPrompt(
             plugin_name=ConversationSummaryPlugin.__name__,
             function_name="SummarizeConversation",
             prompt_template_config=prompt_template_config,
@@ -56,10 +73,13 @@ class ConversationSummaryPlugin:
     ]:
         """Given a long conversation transcript, summarize the conversation.
 
-        :param input: A long conversation transcript.
-        :param kernel: The kernel for function execution.
-        :param arguments: Arguments used by the kernel.
-        :return: KernelArguments with the summarized conversation result in key self.return_key.
+        Args:
+            input (str): A long conversation transcript.
+            kernel (Kernel): The kernel for function execution.
+            arguments (KernelArguments): Arguments used by the kernel.
+
+        Returns:
+            KernelArguments with the summarized conversation result in key self.return_key.
         """
         from semantic_kernel.text import text_chunker
         from semantic_kernel.text.function_extension import aggregate_chunked_results
