@@ -4,7 +4,7 @@ import logging
 from abc import ABC
 from collections.abc import Mapping, Sequence
 from functools import singledispatchmethod
-from typing import TYPE_CHECKING, Any, Literal
+from typing import TYPE_CHECKING, Any, Literal, Protocol, runtime_checkable
 
 from pydantic import Field, field_validator
 
@@ -23,11 +23,25 @@ if TYPE_CHECKING:
     )
     from semantic_kernel.functions.kernel_function import KernelFunction
     from semantic_kernel.functions.types import KERNEL_FUNCTION_TYPE
+    from semantic_kernel.kernel import Kernel
 
 
 logger: logging.Logger = logging.getLogger(__name__)
 
 
+@runtime_checkable
+class AddToKernelCallbackProtocol(Protocol):
+    """Protocol for the callback to be called when the plugin is added to the kernel."""
+
+    def added_to_kernel(self, kernel: "Kernel") -> None:
+        """Called when the plugin is added to the kernel.
+
+        Args:
+            kernel (Kernel): The kernel instance
+        """
+        pass
+
+
 class KernelFunctionExtension(KernelBaseModel, ABC):
     """Kernel function extension."""
 
@@ -54,6 +68,7 @@ class KernelFunctionExtension(KernelBaseModel, ABC):
         parent_directory: str | None = None,
         description: str | None = None,
         class_init_arguments: dict[str, dict[str, Any]] | None = None,
+        encoding: str = "utf-8",
     ) -> "KernelPlugin":
         """Adds a plugin to the kernel's collection of plugins.
 
@@ -66,6 +81,7 @@ class KernelFunctionExtension(KernelBaseModel, ABC):
                 a custom class that contains methods with the kernel_function decorator
                 or a dictionary of functions with the kernel_function decorator for one or
                 several methods.
+                if the custom class has a `added_to_kernel` method, it will be called with the kernel instance.
             plugin_name: The name of the plugin, used if the plugin is not a KernelPlugin,
                 if the plugin is None and the parent_directory is set,
                 KernelPlugin.from_directory is called with those parameters,
@@ -73,6 +89,7 @@ class KernelFunctionExtension(KernelBaseModel, ABC):
             parent_directory: The parent directory path where the plugin directory resides
             description: The description of the plugin, used if the plugin is not a KernelPlugin.
             class_init_arguments: The class initialization arguments
+            encoding: The encoding to use when reading text files. Defaults to "utf-8".
 
         Returns:
             KernelPlugin: The plugin that was added.
@@ -85,13 +102,15 @@ class KernelFunctionExtension(KernelBaseModel, ABC):
             self.plugins[plugin.name] = plugin
             return self.plugins[plugin.name]
         if not plugin_name:
-            raise ValueError("plugin_name must be provided if a plugin is not supplied.")
+            plugin_name = getattr(plugin, "name", plugin.__class__.__name__)
         if not isinstance(plugin_name, str):
             raise TypeError("plugin_name must be a string.")
         if plugin:
             self.plugins[plugin_name] = KernelPlugin.from_object(
                 plugin_name=plugin_name, plugin_instance=plugin, description=description
             )
+            if isinstance(plugin, AddToKernelCallbackProtocol):
+                plugin.added_to_kernel(self)  # type: ignore
             return self.plugins[plugin_name]
         if plugin is None and parent_directory is not None:
             self.plugins[plugin_name] = KernelPlugin.from_directory(
@@ -99,11 +118,12 @@ class KernelFunctionExtension(KernelBaseModel, ABC):
                 parent_directory=parent_directory,
                 description=description,
                 class_init_arguments=class_init_arguments,
+                encoding=encoding,
             )
             return self.plugins[plugin_name]
         raise ValueError("plugin or parent_directory must be provided.")
 
-    def add_plugins(self, plugins: list[KernelPlugin] | dict[str, KernelPlugin | object]) -> None:
+    def add_plugins(self, plugins: list[KernelPlugin | object] | dict[str, KernelPlugin | object]) -> None:
         """Adds a list of plugins to the kernel's collection of plugins.
 
         Args:
@@ -167,7 +187,7 @@ class KernelFunctionExtension(KernelBaseModel, ABC):
             function = KernelFunction.from_prompt(
                 function_name=function_name,
                 plugin_name=plugin_name,
-                description=description,
+                description=description or (prompt_template_config.description if prompt_template_config else None),
                 prompt=prompt,
                 template_format=template_format,
                 prompt_template=prompt_template,
